/*
    Plee The Bear - Level editor

    Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    contact: plee-the-bear@gamned.org

    Please add the tag [PTB] in the subject of your mails.
*/
/**
 * \file bf/spin_ctrl.hpp
 * \brief A spin control that supports any numeric type of value.
 * \author Julien Jorge
 */
#ifndef __BF_SPIN_CTRL_HPP__
#define __BF_SPIN_CTRL_HPP__

#include <wx/spinbutt.h>
#include <wx/textctrl.h>
#include <wx/wx.h>

namespace bf
{
  /**
   * \file bf/spin_ctrl.hpp
   * \brief A spin control that supports any numeric type of value.
   * \author Julien Jorge
   */
  template<typename T>
  class spin_ctrl:
    public wxPanel
  {
  public:
    /** \brief The type of the value stored in the control. */
    typedef T value_type;

    /** \brief The type of the parent class. */
    typedef wxPanel super;

    /** \brief The type of the current class. */
    typedef spin_ctrl<T> self_type;

  public:
    spin_ctrl
    ( wxWindow* parent, wxWindowID id = -1,
      const wxPoint& pos = wxDefaultPosition,
      const wxSize& size = wxDefaultSize,
      long style = 0, value_type min = 0, value_type max = 100,
      value_type initial = 0, value_type step = 1,
      const wxString& name = _T("spin_ctrl") );

    void SetRange( value_type min, value_type max );
    void SetStep( value_type s );

    void SetValue( value_type v );
    value_type GetValue() const;
    wxString GetValueText() const;

  private:
    void BoundValue( value_type v );

    void ValueToText();

    void CreateControls();

    void Up();
    void Down();

    void OnSpinUp( wxSpinEvent& event );
    void OnSpinDown( wxSpinEvent& event );
    void OnChange( wxCommandEvent& event );

  private:
    /** \brief The minimum value. */
    value_type m_min;

    /** \brief The maximum value. */
    value_type m_max;

    /** \brief The current value. */
    value_type m_value;

    /** \brief The step of the spin button. */
    value_type m_step;

    /** \brief The text field in which we display the value. */
    wxTextCtrl* m_text;

    /** \brief The spin button. */
    wxSpinButton* m_spin;

  }; // spin_ctrl
} // namespace bf

#include "bf/impl/spin_ctrl.tpp"

#endif // __BF_SPIN_CTRL_HPP__
