/*
  Plee The Bear

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [PTB] in the subject of your mails.
*/
/**
 * \file base_train.hpp
 * \brief The class describing the base of a train platform
 *        under which Plee is captive.
 * \author Sebastien Angibaud
 */
#ifndef __PTB_BASE_TRAIN_CAPTIVE_HPP__
#define __PTB_BASE_TRAIN_CAPTIVE_HPP__

#include "generic_items/base_train.hpp"
#include "engine/player.hpp"
#include <list>

namespace ptb
{
  /**
   * \brief The class describing a train platform under which Plee is captive.
   * \author Sebastien Angibaud
   */
  class base_train_captive : public bear::base_train
  {
  public:
    /** \brief The type of the parent class. */
    typedef bear::base_train super;

  public:
    base_train_captive();

    virtual void progress( bear::universe::time_type elapsed_time );

    virtual void set_top_contact(bear::universe::physical_item_state& item);

    virtual void move( bear::universe::time_type elapsed_time );

  private:
    /** \brief Indicates the time of the captive. */
    bear::time_ref::time_reference m_captive_time;

    /** \brief The list  of players in the base train. */
    std::list<bear::engine::player*> m_players;

    /** \brief Indicates if the base train had captive Plee. */
    bool m_captive;

    /** \brief Indicates if the captive is ended. */
    bool m_end_captive;

  }; // class base_train_captive
} // namespace ptb

#endif // __PTB_BASE_TRAIN_CAPTIVE_HPP__
